import React, {useEffect, useMemo, useRef, useState} from "react";
import {Button, Modal} from "antd";
import {SortableContext, arrayMove} from '@dnd-kit/sortable';
import {DndContext} from "@dnd-kit/core";

import {VideoListItem} from "@/components/video/video-list-item.tsx";
import {deleteByIds, getList, modifyOrder, playState} from "@/service/api/live.ts";

import {showErrorToast, showToast} from "@/components/message.ts";
import ButtonBatch from "@/components/button-batch.tsx";
import FlvJs from "flv.js";
import {formatDuration} from "@/util/strings.ts";
import {useSetState} from "ahooks";
import {Player, PlayerInstance} from "@/components/video/player.tsx";

const cache: { flvPlayer?: FlvJs.Player, timerPlayNext?: any, timerLoadState?: any, prevUrl?: string } = {}
export default function LiveIndex() {
	const videoRef = useRef<HTMLVideoElement | null>(null)

	const player = useRef<PlayerInstance | null>(null)
	const [videoData, setVideoData] = useState<LiveVideoInfo[]>([])
	const [modal, contextHolder] = Modal.useModal()
	const [checkedIdArray, setCheckedIdArray] = useState<number[]>([])
	const [editable, setEditable] = useState<boolean>(false)

	const [state, setState] = useSetState({
		activeIndex: -1,
		muted: true,
	})
	const activeIndex = useRef(state.activeIndex)
	useEffect(() => {
		activeIndex.current = state.activeIndex
	}, [state.activeIndex])

	const showVideoItem = (index: number) => {
		// 找到对应video item 并显示在视图可见区域
		const container = document.querySelector('.live-video-list-sort-container')
		const item = document.querySelector(`.list-item-${index}`)
		if (item && container) {
			// 获取容器数据
			const containerRect = container.getBoundingClientRect()
			// 获取对应item的数据
			const rect = item.getBoundingClientRect()
			// 计算对应item需要在容器中滚动的距离
			const scrollDistance = rect.top - containerRect.top
			// 设置滚动高度
			container.scrollTo({
				top: index == 0 ? 0 : container.scrollTop + scrollDistance - 10,
				behavior: 'smooth'
			})
		}
	}

	const activeToNext = (index?: number) => {
		const endToFirst = index != undefined && index > -1 ? false : activeIndex.current >= videoData.length - 1
		const _activeIndex = index != undefined && index > -1 ? index : (endToFirst ? 0 : activeIndex.current + 1)
		setState({activeIndex: _activeIndex})
		if (endToFirst) {
			showToast('即将播放第一条视频');
		}
		// 找到对应video item 并显示在视图可见区域
		showVideoItem(_activeIndex)
		return _activeIndex;
	}
	const playVideo = (video: LiveVideoInfo, liveState: LiveState) => {
		if (player.current && video.video_oss_url) {
			if (cache.timerPlayNext) clearTimeout(cache.timerPlayNext)
			const duration = Math.ceil(video.video_duration / 1000)
			const playedTime = (Date.now() / 1000 >> 0) - liveState.live_start_time
			if (playedTime < 0 || playedTime > duration) { // 已播放时间大于总时长了
				//initPlayingState() // 重新获取播放状态
				return;
			}
			player.current?.play(video.video_oss_url, playedTime)
			cache.timerPlayNext = setTimeout(() => {
				const index = activeToNext(), nextVideo = videoData[index]
				playVideo(nextVideo, {live_start_time: (Date.now() / 1000 >> 0), id: nextVideo.id})
			}, (duration - playedTime) * 1000)

		}
	}
	const initPlayingState = () => {
		if (cache.timerLoadState) clearTimeout(cache.timerLoadState)
		if (videoData.length == 0) {
			cache.timerLoadState = setTimeout(initPlayingState, 1000)
			return;
		}
		playState().then(liveState => {
			const video = videoData.find(v => v.id === liveState.id)
			if (video) {
				activeToNext(videoData.findIndex(v => v.id === liveState.id))
				playVideo(video, liveState)
			} else {
				setState({activeIndex: -1})
				cache.timerLoadState = setTimeout(initPlayingState, 5000)
			}
		});
	}
	const clearAllTimer = () => {
		if (cache.timerPlayNext) clearTimeout(cache.timerPlayNext)
		if (cache.timerLoadState) clearTimeout(cache.timerLoadState)
	}

	const loadList = () => {
		clearAllTimer();
		getList().then(res => {
			// console.log('origin list', res.list.map(s => s.id))
			setVideoData(() => (res.list || []))
			setCheckedIdArray([])
		});
	}

	useEffect(initPlayingState, [videoData])
	useEffect(() => {
		loadList()
		return clearAllTimer;
	}, [])

	const processDeleteVideo = async (ids: Id[]) => {
		deleteByIds(ids).then(() => {
			showToast('删除成功!', 'success')
			loadList()
		}).catch(showErrorToast)
	}
	const handleConfirm = () => {
		modal.confirm({
			title: '提示',
			content: '是否采纳移动视频位置操作？',
			onOk: () => {
				//showToast('编辑成功!!!', 'info');
				modifyOrder(videoData.map(s => s.id)).then(() => {
					setEditable(false)
					loadList()
				}).catch(() => {
					showToast('调整视频顺序失败,请重试!')
				})
				// showToast('编辑成功!!!', 'info');
				// console.log('origin list', videoData.map(s => s.id))
			}
		})
	}
	const handleCancelConfirm = () => {
		modal.confirm({
			title: '提示',
			content: '是否取消移动视频位置操作？',
			onOk: () => {
				showToast('退出并清除移动视频位置操作！', 'info');
				loadList()
				setEditable(false)
			},
		})
	}

	const totalDuration = useMemo(() => {
		if (!videoData || videoData.length == 0) return 0;
		// 计算总时长
		return videoData.reduce((sum, v) => sum + Math.ceil(v.video_duration / 1000), 0);
	}, [videoData])

	return (<div className="container py-10 page-live">
		{contextHolder}
		<div className="flex">
			<div className="video-player-container mr-8 flex flex-col">
				<div className="text-center text-base">数字人直播间</div>
				<div className="video-player flex justify-center flex-1 mt-5">
					<div className="live-player relative rounded overflow-hidden w-[360px] h-[636px]"
					     style={{backgroundColor: 'hsl(210, 100%, 48%)'}}>
						<Player ref={player} className="w-[360px] h-[636px] bg-white" muted={true}/>
					</div>
				</div>
				<div className="mt-4 text-center text-sm">
					<span>视频时长: {formatDuration(totalDuration)}</span>
				</div>
			</div>
			<div className="video-list-container flex-1">
				<div className=" bg-white py-8 px-6 rounded">
					<div className="live-control flex justify-between mb-4">
						{editable ? <>
							<div className="flex gap-2">
								<Button type="primary" onClick={handleConfirm}>确定</Button>
								<Button onClick={handleCancelConfirm}>退出</Button>
							</div>
						</> : <div>
							<Button type="primary" onClick={() => setEditable(true)}>重新排序</Button>
						</div>}
						{!editable && <div>
                            <ButtonBatch
                                selected={checkedIdArray}
                                emptyMessage={`请选择要删除的新闻视频`}
                                confirmMessage={`是否删除当前的${checkedIdArray.length}个新闻视频？`}
                                onSuccess={loadList}
                                onProcess={processDeleteVideo}
                            >批量删除</ButtonBatch>
                        </div>}
					</div>
					<div className="live-video-list-sort-container">
						<div className="flex">
							<div className="sort-number-container mr-2">
								{videoData.map((v, index) => (
									<div key={index} className="flex items-center px-2 h-[80px] mt-3 mb-2">
										<div
											className="index-value w-[40px] h-[40px] flex items-center justify-center bg-gray-100 rounded-3xl">{index + 1}</div>
									</div>
								))}
							</div>
							<div className="sort-list-container flex-1">
								<DndContext onDragEnd={(e) => {
									const {active, over} = e;
									if (over && active.id !== over.id) {
										let oldIndex = -1, newIndex = -1;
										setVideoData((items) => {
											oldIndex = items.findIndex(s => s.id == active.id);
											newIndex = items.findIndex(s => s.id == over.id);
											return arrayMove(items, oldIndex, newIndex);
										});
									}
								}}>
									<SortableContext items={videoData}>
										{videoData.map((v, index) => (
											<VideoListItem
												video={v}
												index={index + 1}
												id={v.id}
												key={index}
												active={state.activeIndex == index}
												className={`list-item-${index} mt-3 mb-2`}
												checked={checkedIdArray.includes(v.id)}
												onCheckedChange={(checked) => {
													setCheckedIdArray(idArray => {
														return checked ? idArray.concat(v.id) : idArray.filter(id => id != v.id);
													})
												}}
												onRemove={() => processDeleteVideo([v.id])}
												editable={!editable}
												sortable={editable}
											/>))}
									</SortableContext>
								</DndContext>
							</div>
						</div>
					</div>
				</div>
			</div>
		</div>
	</div>)
}