import {useState} from "react";
import {Checkbox, Empty, Modal, Pagination, Space} from "antd";
import {useRequest} from "ahooks";

import {Card} from "@/components/card";

import SearchPanel from "@/pages/news/components/search-panel.tsx";
import styles from './style.module.scss'
import {getById,getList} from "@/service/api/news.ts";
import {showLoading} from "@/components/message.ts";
import {formatTime} from "@/util/strings.ts";
import ButtonPushNews2Article from "@/pages/news/components/button-push-news2article.tsx";
import ButtonNewsDownload from "@/pages/news/components/button-news-download.tsx";

export default function NewsIndex() {

	const [params, setParams] = useState<ApiArticleSearchParams>({
		pagination: {
			page: 1,
			limit: 10
		}
	})
	const [checkedId, setCheckedId] = useState<Id[]>([])
	const [activeNews, setActiveNews] = useState<NewsInfo>()

	const [state, setState] = useState<{
		checkAll?: boolean;
	}>({})
	const {data} = useRequest(() => getList(params), {
		refreshDeps: [params],
		onSuccess: () => {
			setCheckedId([])
			setState({checkAll:false})
		}
	})

	const handleViewNewsDetail = (id: number) => {
		const {update, close} = showLoading('获取新闻详情...')
		getById(id).then(res => {
			close()
			setActiveNews(res)
		}).catch(() => {
			update('获取新闻详情失败', 'info')
		})
	}


	return (<div className={'container pb-5'}>
		<Card className="search-panel-container my-5">
			<SearchPanel onSearch={setParams}/>
		</Card>
		<Card className="news-list-container">
			{activeNews && <Modal open={true} width={1000} footer={null} onCancel={() => setActiveNews(undefined)}>
                <div className="news-detail px-3 pb-5">
                    <div className="new-title text-2xl">{activeNews?.title}</div>
                    <div className="info mt-2 mb-5 text-sm flex gap-3">
                        <span className="source text-blue-700">{activeNews?.media_name}</span>
                        <span className="create-time text-gray-400">{formatTime(activeNews?.publish_time)}</span>
                    </div>
                    <div className="overflow-auto leading-7 text-base"
                         style={{maxHeight: 1000}} dangerouslySetInnerHTML={{__html: activeNews?.content || ''}}></div>
                </div>
            </Modal>}
			<div className="controls flex justify-between mb-1">
				<div>
					<Checkbox checked={state.checkAll} onChange={e => {
						setState({checkAll: e.target.checked})
						if (e.target.checked) {
							setCheckedId(data?.list?.map(item => item.id) || [])
						} else {
							setCheckedId([])
						}
					}}>全选</Checkbox>
				</div>
				<Space size={10}>
					<ButtonPushNews2Article ids={checkedId}/>
					<ButtonNewsDownload ids={checkedId}/>
				</Space>
			</div>
			<div className={styles.newsList}>
				{data?.list?.map(item => (
					<div key={item.id} className={`py-3 flex items-start border-b border-gray-100 group`}>
						<div
							className={`checkbox mt-[2px] mr-2 ${checkedId.includes(item.id) ? '' : 'opacity-0'} group-hover:opacity-100`}>
							{item.internal_article_id > 0 ? <span className={"inline-block w-[16px] " }></span> :<Checkbox checked={checkedId.includes(item.id)} onChange={() => {
								if (checkedId.includes(item.id)) {
									setCheckedId(checkedId.filter(id => id != item.id))
								} else {
									setCheckedId([...checkedId, item.id])
								}
							}}/> }
						</div>
						<div className="news-content flex-1">
							<div className="flex items-center justify-between">
								<div className="title text-lg cursor-pointer" onClick={() => {
									handleViewNewsDetail(item.id)
								}}>{item.title}</div>
								{item.internal_article_id > 0 &&
                                    <div className="text-sm text-blue-500">已加入编辑界面</div>}
							</div>
							<div className="content flex gap-3 mt-2 mb-3">
								{item.cover && <div
                                    className="cover border border-gray-100 flex items-center rounded overflow-hidden"
                                    style={{width: 100, height: 100}}>
                                    <img className="w-full h-full object-cover" src={item.cover}/>
                                </div>}
								<div className="text text-gray-600 text-sm leading-6 flex-1 text-justify">
									{item.summary}
								</div>
							</div>
							<div className="info text-gray-300 flex items-center justify-between gap-3 text-sm">
								<div>来源: <span>{item.media_name}</span></div>
								{/*<Divider type="vertical" />*/}
								<div>发布时间: <span>{formatTime(item.publish_time)}</span></div>
							</div>
						</div>
					</div>
				))}
			</div>

			{data?.pagination && data?.pagination.total > 0 ? <div className="flex justify-center mt-10">
				<Pagination
					current={params.pagination.page}
					total={data?.pagination.total}
					pageSize={data?.pagination.limit}
					showSizeChanger={false}
					simple={true}
					rootClassName={'simple-pagination'}
					onChange={(page) => setParams(prev=>({...prev,pagination: {page, limit: 10}}))}
				/>
			</div> : <div className="py-10">
				<Empty />
			</div>
			}
		</Card>
	</div>)
}