import {Empty, Modal} from "antd";
import React, {useEffect, useMemo, useRef, useState} from "react";
import {DndContext} from "@dnd-kit/core";
import {arrayMove, SortableContext} from "@dnd-kit/sortable";
import {useSetState} from "ahooks";
import {CheckCircleFilled} from "@ant-design/icons";
import {clsx} from "clsx";

import {VideoListItem} from "@/components/video/video-list-item.tsx";
import ArticleEditModal from "@/components/article/edit-modal.tsx";
import {deleteByIds, getList, modifyOrder, VideoStatus} from "@/service/api/video.ts";
import {formatDuration} from "@/util/strings.ts";
import ButtonBatch from "@/components/button-batch.tsx";
import {showToast} from "@/components/message.ts";
import {Player, PlayerInstance} from "@/components/video/player.tsx";
import ButtonPush2Room from "@/pages/video/components/button-push2room.tsx";

export default function VideoIndex() {
	const [editId, setEditId] = useState(-1)
	const [videoData, setVideoData] = useState<VideoInfo[]>([])
	const [modal, contextHolder] = Modal.useModal()
	const player = useRef<PlayerInstance|null>(null)
	const [state, setState] = useSetState({
		checkedAll: false,
		playingIndex: -1,
	})
	const [checkedIdArray, setCheckedIdArray] = useState<Id[]>([])

	// 加载列表
	const loadList = (needReset = true) => {
		getList().then((ret) => {
			const list = ret.list || []
			setVideoData(list)
			if (needReset) {
				setCheckedIdArray([])
				setState({checkedAll: false, playingIndex: -1})
			}
			// 判断是否有生成中的视频
			if (list.filter(s => s.status == VideoStatus.Generating).length > 0) {
				// 每5s重新获取一次最新数据
				setTimeout(() => loadList(false), 5000)
			}
		})
	}

	// 播放视频
	const playVideo = (video: VideoInfo, playingIndex: number) => {
		if (video.oss_video_url && video.status !== 1) {
			setState({playingIndex})
			player.current?.play(video.oss_video_url, 0)
		}
	}
	// 处理全选
	const handleAllCheckedChange = () => {
		setCheckedIdArray(state.checkedAll ? [] : videoData.map(v => v.id))
		setState({
			checkedAll: !state.checkedAll
		})
	}
	const handleModifySort = () => {
		setVideoData((items) => {
			modifyOrder(items.map(s => s.id)).catch(() => {
				showToast('调整视频顺序失败,请重试!')
			}).finally(loadList)
			return items;
		})
	}
	//
	useEffect(loadList, [])
	const totalDuration = useMemo(() => {
		if (!videoData || videoData.length == 0) return 0;
		// 计算总时长
		return videoData.reduce((sum, v) => sum + Math.ceil(v.duration / 1000), 0);
	}, [videoData])

	return (<div className="container py-10 page-live">
		{contextHolder}
		<div className="flex">
			<div className="video-list-container bg-white p-10 rounded flex flex-col flex-1">
				<div className="live-control flex justify-between mb-5">
					<div className="pl-[70px]">
						<span>视频时长: {formatDuration(totalDuration)}</span>
					</div>
					<div className="flex gap-2 items-center pr-[10px]">
						<ButtonBatch
							onProcess={deleteByIds}
							selected={checkedIdArray}
							emptyMessage={`请选择要删除的新闻视频`}
							confirmMessage={`是否删除当前的${checkedIdArray.length}个新闻视频？`}
							onSuccess={() => {
								showToast('删除成功!', 'success')
								loadList()
							}}
						>批量删除</ButtonBatch>

						<button className="ml-5 hover:text-blue-300 text-gray-400 text-lg"
						        onClick={handleAllCheckedChange}>
							<CheckCircleFilled className={clsx({'text-blue-500': state.checkedAll})}/>
						</button>
					</div>
				</div>
				<div className={'video-list-sort-container flex-1'}>
					<div className="flex my-2">
						{videoData.length == 0 ? <div className="m-auto"><Empty/></div> : <>
							<div className="sort-number-container mr-2">
								{videoData.map((v, index) => (
									<div key={index} className="flex items-center px-2 h-[80px] mt-3 mb-2">
										<div
											className="index-value w-[40px] h-[40px] flex items-center justify-center bg-gray-100 rounded-3xl">{index + 1}</div>
									</div>
								))}
							</div>
							<div className="sort-list-container flex-1">
								<DndContext onDragEnd={(e) => {
									const {active, over} = e;
									if (over && active.id !== over.id) {
										let oldIndex = -1, newIndex = -1;
										const originArr = [...videoData]
										console.log(originArr.map(s => s.id))
										setVideoData((items) => {
											oldIndex = items.findIndex(s => s.id == active.id);
											newIndex = items.findIndex(s => s.id == over.id);
											return arrayMove(items, oldIndex, newIndex);
										});
										modal.confirm({
											title: '提示',
											content: '是否要移动到指定位置',
											onOk: handleModifySort,
											onCancel: () => {
												setVideoData(originArr);
											}
										})
									}
								}}>
									<SortableContext items={videoData}>
										{videoData.map((v, index) => (
											<VideoListItem
												video={v}
												index={index + 1}
												id={v.id}
												key={index}
												type={'create'}
												active={state.playingIndex == index}
												checked={checkedIdArray.includes(v.id)}
												className={`list-item-${index} mt-3 mb-2`}
												onCheckedChange={(checked) => {
													setCheckedIdArray(idArray => {
														const newArr = checked ? idArray.concat(v.id) : idArray.filter(id => id != v.id);
														setState({checkedAll: newArr.length == videoData.length})
														return newArr;
													})
												}}
												onPlay={v.status == VideoStatus.Generating ? undefined :() => playVideo(v, index)}
												onEdit={v.status == VideoStatus.Generating ? undefined : () => {
													setEditId(v.article_id)
												}}
												editable={v.status != VideoStatus.Generating}
												sortable={v.status != VideoStatus.Generating}
											/>))}
									</SortableContext>
								</DndContext>
							</div>
						</>}
					</div>
				</div>
				<div className="text-right mt-5">
					{/*<ButtonBatch*/}
					{/*	type={'primary'}*/}
					{/*	onProcess={push2room}*/}
					{/*	selected={checkedIdArray}*/}
					{/*	emptyMessage={`请选择要推流的新闻`}*/}
					{/*	confirmMessage={`是否确定一键推流选中新闻视频？`}*/}
					{/*	onSuccess={loadList}*/}
					{/*>一键推流</ButtonBatch>*/}
					<ButtonPush2Room ids={checkedIdArray} list={videoData} onSuccess={loadList}/>
				</div>
			</div>
			<div className="video-player-container ml-16 w-[360px] flex flex-col">
				<div className="text-center text-base">预览视频</div>
				<div className="video-player flex items-center mt-2">
					<div className=" w-[360px] h-[630px] rounded overflow-hidden">
						{/*<video ref={videoRef} poster={videoData[state.playingIndex]?.cover}  preload="auto" playsinline webkit-playsinline className="w-full bg-white w-[360px] h-[640px]"></video>*/}
						<Player
							ref={player} url={videoData[state.playingIndex]?.oss_video_url}
							onChange={(state) => {
								if (state.end || state.end) setState({playingIndex: -1})
							}}
							className="w-[360px] h-[640px] bg-white"/>
					</div>
				</div>
			</div>
		</div>
		<ArticleEditModal type={'video'} id={editId} onClose={() => setEditId(-1)}/>
	</div>)
}